package AntiCaptcha;

use strict;
use IO::Socket::INET;
use Carp qw(croak);

use constant
{
    CRLF => qq/\015\012/,
    BOUNDARY => q/Asrf456BGe4h/
};

my %error_codes =
(
    ERROR_WRONG_USER_KEY            =>  { msg => q/Wrong AntiCaptcha key/,      is_fatal => 1 },
    ERROR_KEY_DOES_NOT_EXIST        =>  { msg => q/Wrong AntiCaptcha key/,      is_fatal => 1 },
    ERROR_TOO_BIG_CAPTCHA_FILESIZE  =>  { msg => q/Too big image file/,         is_fatal => 0 },
    ERROR_ZERO_CAPTCHA_FILESIZE     =>  { msg => q/Zero size image file/,       is_fatal => 0 },
    ERROR_WRONG_FILE_EXTENSION      =>  { msg => q/Wrong image file extension/, is_fatal => 0 },
    ERROR_NO_SLOT_AVAILABLE         =>  { msg => q/No slot available/,          is_fatal => 0 },
    CAPCHA_NOT_READY                =>  { msg => q/Captcha not ready/,          is_fatal => 0 },
    ERROR_WRONG_ID_FORMAT           =>  { msg => q/Wrong captcha id format/,    is_fatal => 1 },
    ERROR_NO_SUCH_CAPCHA_ID         =>  { msg => q/No such captcha id/,         is_fatal => 1 }
);

my %req_var_filter =
(
    verbose => 1,
    is_fatal => 1,
    host => 1
);

sub new
{
    my $self = shift;
    
    my $obj = bless 
    {
        verbose => 1,
        phrase => 0,
        regsense => 0,
        numeric => 0,
        min_len => 0,
        max_len => 0,
        is_russian => 2,
        is_fatal => 0
    }, $self;
    
    croak 'Please specify AntiCaptcha host'.CRLF unless $_[0];
    croak 'Please specify AntiCaptcha key'.CRLF unless $_[1];
 
    ($obj->{host}, $obj->{key}) = @_;

    return $obj;
}

sub sock 
{
    my $self = shift;
    
    my $sock = IO::Socket::INET->new
    (
        PeerAddr => $self->{host},
        PeerPort => 80, 
        PeerProto => 'tcp', 
        TimeOut => 10,
    ) or _warn(q/Can't connect to AntiCaptcha/);
 
    return $sock;
}

sub recognize
{
    my ($self, $image, $type) = @_;
    
    my $image_data = $type ? $image : _read_file($image);
    return 0 unless $image_data;
    
    my %request_data =
    (
        method => 'post',
        soft_id => 7,
        file => $image_data
    );
    
    while( my ($k, $v) = each %{$self} )
    {
        next if $req_var_filter{$k};
        $request_data{$k} = $v;
    }
    
    my $request_body = '--'.BOUNDARY.CRLF;
    my @keys = keys %request_data;
    
    for my $k(@keys)
    {
        $request_body.=
            'Content-Disposition: form-data; name="'.$k.'"'.($k eq 'file' ? '; filename="image"'.CRLF : CRLF.CRLF).
            ($k eq 'file' ? 'Content-Type: image'.CRLF.CRLF : '').
            $request_data{$k}.
            CRLF.
            '--'.BOUNDARY.
            ($k eq $keys[$#keys] ? '--' : '').
            CRLF
    }
    
    
    my $request =
    'POST /in.php HTTP/1.0'.CRLF.
    'Host: '.$self->{host}.CRLF.
    'Content-Type: multipart/form-data; boundary='.BOUNDARY.CRLF.
    'Content-Length: '.length($request_body).CRLF.CRLF.$request_body;
    
    my $sock = $self->sock() or return 0;
    
    syswrite $sock, $request;
    sysread $sock, my $answ, 1024;
    close $sock;
    
    
    return $self->_handle_response($answ);
}

sub get_status
{
    my ($self, $id) = @_;

    croak 'Specified ID must be numeric' unless $id =~ /^\d+/;
    
    my $request =
    'GET /res.php?key='.$self->{'key'}.'&action=get&id='.$id.' HTTP/1.0'.CRLF.
    'Host: '.$self->{'host'}.CRLF.CRLF;
 
    my $sock = $self->sock() or return 0;
    
    syswrite $sock, $request;
    sysread $sock, my $answ, 1024;
    close $sock;
 
    return $self->_handle_response($answ);
}


sub report
{
    my ($self, $id) = @_;

    croak 'Specified ID must be numeric' unless $id =~ /^\d+$/;
    
    my $request =
    'GET /res.php?key='.$self->{key}.'&action=reportbad&id='.$id.' HTTP/1.0'.CRLF.
    'Host: '.$self->{host}.CRLF.CRLF;
 
    my $sock = $self->sock() or return 0;
    syswrite $sock, $request;
    close $sock;
    
    return 1;
}

sub _handle_response
{
    my ($self, $response) = @_;
    
    if($response =~ /\r\n\r\n(.+)/s)
    {
        chomp(my $code = $1);
            
        if($code =~ /OK\|(\d+)/)
        {
            return $1;
        }
        elsif($error_codes{$code})
        {
            $self->_warn_if($error_codes{$code});
        }
    }
    else
    {
        $self->_warn_if( {msg => q/Something wrong with response/, is_fatal => 1} );
    }
    
    return 0;
}

sub _read_file
{
    local @ARGV = shift;
    local $/ = wantarray ? $/ : undef;
    <>;
}

sub _warn
{
    ($_) = @_;
    warn $_.CRLF;
}

sub _warn_if
{
    my ($self, $msg_inf) = @_;
    
    if($self->{is_fatal} && $msg_inf->{is_fatal})
    {
        die $msg_inf->{msg};
    }
    elsif($self->{verbose})
    {
        _warn($msg_inf->{msg});
    }
}

1;
